# Generated from Python3Parser.g4 by ANTLR 4.11.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .Python3Parser import Python3Parser
else:
    from pyutplugins.ioplugins.python.pyantlrparser.Python3Parser import Python3Parser

# This class defines a complete listener for a parse tree produced by Python3Parser.
class Python3ParserListener(ParseTreeListener):

    # Enter a parse tree produced by Python3Parser#single_input.
    def enterSingle_input(self, ctx:Python3Parser.Single_inputContext):
        pass

    # Exit a parse tree produced by Python3Parser#single_input.
    def exitSingle_input(self, ctx:Python3Parser.Single_inputContext):
        pass


    # Enter a parse tree produced by Python3Parser#file_input.
    def enterFile_input(self, ctx:Python3Parser.File_inputContext):
        pass

    # Exit a parse tree produced by Python3Parser#file_input.
    def exitFile_input(self, ctx:Python3Parser.File_inputContext):
        pass


    # Enter a parse tree produced by Python3Parser#eval_input.
    def enterEval_input(self, ctx:Python3Parser.Eval_inputContext):
        pass

    # Exit a parse tree produced by Python3Parser#eval_input.
    def exitEval_input(self, ctx:Python3Parser.Eval_inputContext):
        pass


    # Enter a parse tree produced by Python3Parser#decorator.
    def enterDecorator(self, ctx:Python3Parser.DecoratorContext):
        pass

    # Exit a parse tree produced by Python3Parser#decorator.
    def exitDecorator(self, ctx:Python3Parser.DecoratorContext):
        pass


    # Enter a parse tree produced by Python3Parser#decorators.
    def enterDecorators(self, ctx:Python3Parser.DecoratorsContext):
        pass

    # Exit a parse tree produced by Python3Parser#decorators.
    def exitDecorators(self, ctx:Python3Parser.DecoratorsContext):
        pass


    # Enter a parse tree produced by Python3Parser#decorated.
    def enterDecorated(self, ctx:Python3Parser.DecoratedContext):
        pass

    # Exit a parse tree produced by Python3Parser#decorated.
    def exitDecorated(self, ctx:Python3Parser.DecoratedContext):
        pass


    # Enter a parse tree produced by Python3Parser#async_funcdef.
    def enterAsync_funcdef(self, ctx:Python3Parser.Async_funcdefContext):
        pass

    # Exit a parse tree produced by Python3Parser#async_funcdef.
    def exitAsync_funcdef(self, ctx:Python3Parser.Async_funcdefContext):
        pass


    # Enter a parse tree produced by Python3Parser#funcdef.
    def enterFuncdef(self, ctx:Python3Parser.FuncdefContext):
        pass

    # Exit a parse tree produced by Python3Parser#funcdef.
    def exitFuncdef(self, ctx:Python3Parser.FuncdefContext):
        pass


    # Enter a parse tree produced by Python3Parser#parameters.
    def enterParameters(self, ctx:Python3Parser.ParametersContext):
        pass

    # Exit a parse tree produced by Python3Parser#parameters.
    def exitParameters(self, ctx:Python3Parser.ParametersContext):
        pass


    # Enter a parse tree produced by Python3Parser#typedargslist.
    def enterTypedargslist(self, ctx:Python3Parser.TypedargslistContext):
        pass

    # Exit a parse tree produced by Python3Parser#typedargslist.
    def exitTypedargslist(self, ctx:Python3Parser.TypedargslistContext):
        pass


    # Enter a parse tree produced by Python3Parser#tfpdef.
    def enterTfpdef(self, ctx:Python3Parser.TfpdefContext):
        pass

    # Exit a parse tree produced by Python3Parser#tfpdef.
    def exitTfpdef(self, ctx:Python3Parser.TfpdefContext):
        pass


    # Enter a parse tree produced by Python3Parser#varargslist.
    def enterVarargslist(self, ctx:Python3Parser.VarargslistContext):
        pass

    # Exit a parse tree produced by Python3Parser#varargslist.
    def exitVarargslist(self, ctx:Python3Parser.VarargslistContext):
        pass


    # Enter a parse tree produced by Python3Parser#vfpdef.
    def enterVfpdef(self, ctx:Python3Parser.VfpdefContext):
        pass

    # Exit a parse tree produced by Python3Parser#vfpdef.
    def exitVfpdef(self, ctx:Python3Parser.VfpdefContext):
        pass


    # Enter a parse tree produced by Python3Parser#stmt.
    def enterStmt(self, ctx:Python3Parser.StmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#stmt.
    def exitStmt(self, ctx:Python3Parser.StmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#simple_stmts.
    def enterSimple_stmts(self, ctx:Python3Parser.Simple_stmtsContext):
        pass

    # Exit a parse tree produced by Python3Parser#simple_stmts.
    def exitSimple_stmts(self, ctx:Python3Parser.Simple_stmtsContext):
        pass


    # Enter a parse tree produced by Python3Parser#simple_stmt.
    def enterSimple_stmt(self, ctx:Python3Parser.Simple_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#simple_stmt.
    def exitSimple_stmt(self, ctx:Python3Parser.Simple_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#expr_stmt.
    def enterExpr_stmt(self, ctx:Python3Parser.Expr_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#expr_stmt.
    def exitExpr_stmt(self, ctx:Python3Parser.Expr_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#annassign.
    def enterAnnassign(self, ctx:Python3Parser.AnnassignContext):
        pass

    # Exit a parse tree produced by Python3Parser#annassign.
    def exitAnnassign(self, ctx:Python3Parser.AnnassignContext):
        pass


    # Enter a parse tree produced by Python3Parser#testlist_star_expr.
    def enterTestlist_star_expr(self, ctx:Python3Parser.Testlist_star_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#testlist_star_expr.
    def exitTestlist_star_expr(self, ctx:Python3Parser.Testlist_star_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#augassign.
    def enterAugassign(self, ctx:Python3Parser.AugassignContext):
        pass

    # Exit a parse tree produced by Python3Parser#augassign.
    def exitAugassign(self, ctx:Python3Parser.AugassignContext):
        pass


    # Enter a parse tree produced by Python3Parser#del_stmt.
    def enterDel_stmt(self, ctx:Python3Parser.Del_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#del_stmt.
    def exitDel_stmt(self, ctx:Python3Parser.Del_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#pass_stmt.
    def enterPass_stmt(self, ctx:Python3Parser.Pass_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#pass_stmt.
    def exitPass_stmt(self, ctx:Python3Parser.Pass_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#flow_stmt.
    def enterFlow_stmt(self, ctx:Python3Parser.Flow_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#flow_stmt.
    def exitFlow_stmt(self, ctx:Python3Parser.Flow_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#break_stmt.
    def enterBreak_stmt(self, ctx:Python3Parser.Break_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#break_stmt.
    def exitBreak_stmt(self, ctx:Python3Parser.Break_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#continue_stmt.
    def enterContinue_stmt(self, ctx:Python3Parser.Continue_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#continue_stmt.
    def exitContinue_stmt(self, ctx:Python3Parser.Continue_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#return_stmt.
    def enterReturn_stmt(self, ctx:Python3Parser.Return_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#return_stmt.
    def exitReturn_stmt(self, ctx:Python3Parser.Return_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#yield_stmt.
    def enterYield_stmt(self, ctx:Python3Parser.Yield_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#yield_stmt.
    def exitYield_stmt(self, ctx:Python3Parser.Yield_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#raise_stmt.
    def enterRaise_stmt(self, ctx:Python3Parser.Raise_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#raise_stmt.
    def exitRaise_stmt(self, ctx:Python3Parser.Raise_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#import_stmt.
    def enterImport_stmt(self, ctx:Python3Parser.Import_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#import_stmt.
    def exitImport_stmt(self, ctx:Python3Parser.Import_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#import_name.
    def enterImport_name(self, ctx:Python3Parser.Import_nameContext):
        pass

    # Exit a parse tree produced by Python3Parser#import_name.
    def exitImport_name(self, ctx:Python3Parser.Import_nameContext):
        pass


    # Enter a parse tree produced by Python3Parser#import_from.
    def enterImport_from(self, ctx:Python3Parser.Import_fromContext):
        pass

    # Exit a parse tree produced by Python3Parser#import_from.
    def exitImport_from(self, ctx:Python3Parser.Import_fromContext):
        pass


    # Enter a parse tree produced by Python3Parser#import_as_name.
    def enterImport_as_name(self, ctx:Python3Parser.Import_as_nameContext):
        pass

    # Exit a parse tree produced by Python3Parser#import_as_name.
    def exitImport_as_name(self, ctx:Python3Parser.Import_as_nameContext):
        pass


    # Enter a parse tree produced by Python3Parser#dotted_as_name.
    def enterDotted_as_name(self, ctx:Python3Parser.Dotted_as_nameContext):
        pass

    # Exit a parse tree produced by Python3Parser#dotted_as_name.
    def exitDotted_as_name(self, ctx:Python3Parser.Dotted_as_nameContext):
        pass


    # Enter a parse tree produced by Python3Parser#import_as_names.
    def enterImport_as_names(self, ctx:Python3Parser.Import_as_namesContext):
        pass

    # Exit a parse tree produced by Python3Parser#import_as_names.
    def exitImport_as_names(self, ctx:Python3Parser.Import_as_namesContext):
        pass


    # Enter a parse tree produced by Python3Parser#dotted_as_names.
    def enterDotted_as_names(self, ctx:Python3Parser.Dotted_as_namesContext):
        pass

    # Exit a parse tree produced by Python3Parser#dotted_as_names.
    def exitDotted_as_names(self, ctx:Python3Parser.Dotted_as_namesContext):
        pass


    # Enter a parse tree produced by Python3Parser#dotted_name.
    def enterDotted_name(self, ctx:Python3Parser.Dotted_nameContext):
        pass

    # Exit a parse tree produced by Python3Parser#dotted_name.
    def exitDotted_name(self, ctx:Python3Parser.Dotted_nameContext):
        pass


    # Enter a parse tree produced by Python3Parser#global_stmt.
    def enterGlobal_stmt(self, ctx:Python3Parser.Global_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#global_stmt.
    def exitGlobal_stmt(self, ctx:Python3Parser.Global_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#nonlocal_stmt.
    def enterNonlocal_stmt(self, ctx:Python3Parser.Nonlocal_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#nonlocal_stmt.
    def exitNonlocal_stmt(self, ctx:Python3Parser.Nonlocal_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#assert_stmt.
    def enterAssert_stmt(self, ctx:Python3Parser.Assert_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#assert_stmt.
    def exitAssert_stmt(self, ctx:Python3Parser.Assert_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#compound_stmt.
    def enterCompound_stmt(self, ctx:Python3Parser.Compound_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#compound_stmt.
    def exitCompound_stmt(self, ctx:Python3Parser.Compound_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#async_stmt.
    def enterAsync_stmt(self, ctx:Python3Parser.Async_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#async_stmt.
    def exitAsync_stmt(self, ctx:Python3Parser.Async_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#if_stmt.
    def enterIf_stmt(self, ctx:Python3Parser.If_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#if_stmt.
    def exitIf_stmt(self, ctx:Python3Parser.If_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#while_stmt.
    def enterWhile_stmt(self, ctx:Python3Parser.While_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#while_stmt.
    def exitWhile_stmt(self, ctx:Python3Parser.While_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#for_stmt.
    def enterFor_stmt(self, ctx:Python3Parser.For_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#for_stmt.
    def exitFor_stmt(self, ctx:Python3Parser.For_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#try_stmt.
    def enterTry_stmt(self, ctx:Python3Parser.Try_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#try_stmt.
    def exitTry_stmt(self, ctx:Python3Parser.Try_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#with_stmt.
    def enterWith_stmt(self, ctx:Python3Parser.With_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#with_stmt.
    def exitWith_stmt(self, ctx:Python3Parser.With_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#with_item.
    def enterWith_item(self, ctx:Python3Parser.With_itemContext):
        pass

    # Exit a parse tree produced by Python3Parser#with_item.
    def exitWith_item(self, ctx:Python3Parser.With_itemContext):
        pass


    # Enter a parse tree produced by Python3Parser#except_clause.
    def enterExcept_clause(self, ctx:Python3Parser.Except_clauseContext):
        pass

    # Exit a parse tree produced by Python3Parser#except_clause.
    def exitExcept_clause(self, ctx:Python3Parser.Except_clauseContext):
        pass


    # Enter a parse tree produced by Python3Parser#block.
    def enterBlock(self, ctx:Python3Parser.BlockContext):
        pass

    # Exit a parse tree produced by Python3Parser#block.
    def exitBlock(self, ctx:Python3Parser.BlockContext):
        pass


    # Enter a parse tree produced by Python3Parser#match_stmt.
    def enterMatch_stmt(self, ctx:Python3Parser.Match_stmtContext):
        pass

    # Exit a parse tree produced by Python3Parser#match_stmt.
    def exitMatch_stmt(self, ctx:Python3Parser.Match_stmtContext):
        pass


    # Enter a parse tree produced by Python3Parser#subject_expr.
    def enterSubject_expr(self, ctx:Python3Parser.Subject_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#subject_expr.
    def exitSubject_expr(self, ctx:Python3Parser.Subject_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#star_named_expressions.
    def enterStar_named_expressions(self, ctx:Python3Parser.Star_named_expressionsContext):
        pass

    # Exit a parse tree produced by Python3Parser#star_named_expressions.
    def exitStar_named_expressions(self, ctx:Python3Parser.Star_named_expressionsContext):
        pass


    # Enter a parse tree produced by Python3Parser#star_named_expression.
    def enterStar_named_expression(self, ctx:Python3Parser.Star_named_expressionContext):
        pass

    # Exit a parse tree produced by Python3Parser#star_named_expression.
    def exitStar_named_expression(self, ctx:Python3Parser.Star_named_expressionContext):
        pass


    # Enter a parse tree produced by Python3Parser#case_block.
    def enterCase_block(self, ctx:Python3Parser.Case_blockContext):
        pass

    # Exit a parse tree produced by Python3Parser#case_block.
    def exitCase_block(self, ctx:Python3Parser.Case_blockContext):
        pass


    # Enter a parse tree produced by Python3Parser#guard.
    def enterGuard(self, ctx:Python3Parser.GuardContext):
        pass

    # Exit a parse tree produced by Python3Parser#guard.
    def exitGuard(self, ctx:Python3Parser.GuardContext):
        pass


    # Enter a parse tree produced by Python3Parser#patterns.
    def enterPatterns(self, ctx:Python3Parser.PatternsContext):
        pass

    # Exit a parse tree produced by Python3Parser#patterns.
    def exitPatterns(self, ctx:Python3Parser.PatternsContext):
        pass


    # Enter a parse tree produced by Python3Parser#pattern.
    def enterPattern(self, ctx:Python3Parser.PatternContext):
        pass

    # Exit a parse tree produced by Python3Parser#pattern.
    def exitPattern(self, ctx:Python3Parser.PatternContext):
        pass


    # Enter a parse tree produced by Python3Parser#as_pattern.
    def enterAs_pattern(self, ctx:Python3Parser.As_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#as_pattern.
    def exitAs_pattern(self, ctx:Python3Parser.As_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#or_pattern.
    def enterOr_pattern(self, ctx:Python3Parser.Or_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#or_pattern.
    def exitOr_pattern(self, ctx:Python3Parser.Or_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#closed_pattern.
    def enterClosed_pattern(self, ctx:Python3Parser.Closed_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#closed_pattern.
    def exitClosed_pattern(self, ctx:Python3Parser.Closed_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#literal_pattern.
    def enterLiteral_pattern(self, ctx:Python3Parser.Literal_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#literal_pattern.
    def exitLiteral_pattern(self, ctx:Python3Parser.Literal_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#literal_expr.
    def enterLiteral_expr(self, ctx:Python3Parser.Literal_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#literal_expr.
    def exitLiteral_expr(self, ctx:Python3Parser.Literal_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#complex_number.
    def enterComplex_number(self, ctx:Python3Parser.Complex_numberContext):
        pass

    # Exit a parse tree produced by Python3Parser#complex_number.
    def exitComplex_number(self, ctx:Python3Parser.Complex_numberContext):
        pass


    # Enter a parse tree produced by Python3Parser#signed_number.
    def enterSigned_number(self, ctx:Python3Parser.Signed_numberContext):
        pass

    # Exit a parse tree produced by Python3Parser#signed_number.
    def exitSigned_number(self, ctx:Python3Parser.Signed_numberContext):
        pass


    # Enter a parse tree produced by Python3Parser#signed_real_number.
    def enterSigned_real_number(self, ctx:Python3Parser.Signed_real_numberContext):
        pass

    # Exit a parse tree produced by Python3Parser#signed_real_number.
    def exitSigned_real_number(self, ctx:Python3Parser.Signed_real_numberContext):
        pass


    # Enter a parse tree produced by Python3Parser#real_number.
    def enterReal_number(self, ctx:Python3Parser.Real_numberContext):
        pass

    # Exit a parse tree produced by Python3Parser#real_number.
    def exitReal_number(self, ctx:Python3Parser.Real_numberContext):
        pass


    # Enter a parse tree produced by Python3Parser#imaginary_number.
    def enterImaginary_number(self, ctx:Python3Parser.Imaginary_numberContext):
        pass

    # Exit a parse tree produced by Python3Parser#imaginary_number.
    def exitImaginary_number(self, ctx:Python3Parser.Imaginary_numberContext):
        pass


    # Enter a parse tree produced by Python3Parser#capture_pattern.
    def enterCapture_pattern(self, ctx:Python3Parser.Capture_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#capture_pattern.
    def exitCapture_pattern(self, ctx:Python3Parser.Capture_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#pattern_capture_target.
    def enterPattern_capture_target(self, ctx:Python3Parser.Pattern_capture_targetContext):
        pass

    # Exit a parse tree produced by Python3Parser#pattern_capture_target.
    def exitPattern_capture_target(self, ctx:Python3Parser.Pattern_capture_targetContext):
        pass


    # Enter a parse tree produced by Python3Parser#wildcard_pattern.
    def enterWildcard_pattern(self, ctx:Python3Parser.Wildcard_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#wildcard_pattern.
    def exitWildcard_pattern(self, ctx:Python3Parser.Wildcard_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#value_pattern.
    def enterValue_pattern(self, ctx:Python3Parser.Value_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#value_pattern.
    def exitValue_pattern(self, ctx:Python3Parser.Value_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#attr.
    def enterAttr(self, ctx:Python3Parser.AttrContext):
        pass

    # Exit a parse tree produced by Python3Parser#attr.
    def exitAttr(self, ctx:Python3Parser.AttrContext):
        pass


    # Enter a parse tree produced by Python3Parser#name_or_attr.
    def enterName_or_attr(self, ctx:Python3Parser.Name_or_attrContext):
        pass

    # Exit a parse tree produced by Python3Parser#name_or_attr.
    def exitName_or_attr(self, ctx:Python3Parser.Name_or_attrContext):
        pass


    # Enter a parse tree produced by Python3Parser#group_pattern.
    def enterGroup_pattern(self, ctx:Python3Parser.Group_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#group_pattern.
    def exitGroup_pattern(self, ctx:Python3Parser.Group_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#sequence_pattern.
    def enterSequence_pattern(self, ctx:Python3Parser.Sequence_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#sequence_pattern.
    def exitSequence_pattern(self, ctx:Python3Parser.Sequence_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#open_sequence_pattern.
    def enterOpen_sequence_pattern(self, ctx:Python3Parser.Open_sequence_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#open_sequence_pattern.
    def exitOpen_sequence_pattern(self, ctx:Python3Parser.Open_sequence_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#maybe_sequence_pattern.
    def enterMaybe_sequence_pattern(self, ctx:Python3Parser.Maybe_sequence_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#maybe_sequence_pattern.
    def exitMaybe_sequence_pattern(self, ctx:Python3Parser.Maybe_sequence_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#maybe_star_pattern.
    def enterMaybe_star_pattern(self, ctx:Python3Parser.Maybe_star_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#maybe_star_pattern.
    def exitMaybe_star_pattern(self, ctx:Python3Parser.Maybe_star_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#star_pattern.
    def enterStar_pattern(self, ctx:Python3Parser.Star_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#star_pattern.
    def exitStar_pattern(self, ctx:Python3Parser.Star_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#mapping_pattern.
    def enterMapping_pattern(self, ctx:Python3Parser.Mapping_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#mapping_pattern.
    def exitMapping_pattern(self, ctx:Python3Parser.Mapping_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#items_pattern.
    def enterItems_pattern(self, ctx:Python3Parser.Items_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#items_pattern.
    def exitItems_pattern(self, ctx:Python3Parser.Items_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#key_value_pattern.
    def enterKey_value_pattern(self, ctx:Python3Parser.Key_value_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#key_value_pattern.
    def exitKey_value_pattern(self, ctx:Python3Parser.Key_value_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#double_star_pattern.
    def enterDouble_star_pattern(self, ctx:Python3Parser.Double_star_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#double_star_pattern.
    def exitDouble_star_pattern(self, ctx:Python3Parser.Double_star_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#class_pattern.
    def enterClass_pattern(self, ctx:Python3Parser.Class_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#class_pattern.
    def exitClass_pattern(self, ctx:Python3Parser.Class_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#positional_patterns.
    def enterPositional_patterns(self, ctx:Python3Parser.Positional_patternsContext):
        pass

    # Exit a parse tree produced by Python3Parser#positional_patterns.
    def exitPositional_patterns(self, ctx:Python3Parser.Positional_patternsContext):
        pass


    # Enter a parse tree produced by Python3Parser#keyword_patterns.
    def enterKeyword_patterns(self, ctx:Python3Parser.Keyword_patternsContext):
        pass

    # Exit a parse tree produced by Python3Parser#keyword_patterns.
    def exitKeyword_patterns(self, ctx:Python3Parser.Keyword_patternsContext):
        pass


    # Enter a parse tree produced by Python3Parser#keyword_pattern.
    def enterKeyword_pattern(self, ctx:Python3Parser.Keyword_patternContext):
        pass

    # Exit a parse tree produced by Python3Parser#keyword_pattern.
    def exitKeyword_pattern(self, ctx:Python3Parser.Keyword_patternContext):
        pass


    # Enter a parse tree produced by Python3Parser#test.
    def enterTest(self, ctx:Python3Parser.TestContext):
        pass

    # Exit a parse tree produced by Python3Parser#test.
    def exitTest(self, ctx:Python3Parser.TestContext):
        pass


    # Enter a parse tree produced by Python3Parser#test_nocond.
    def enterTest_nocond(self, ctx:Python3Parser.Test_nocondContext):
        pass

    # Exit a parse tree produced by Python3Parser#test_nocond.
    def exitTest_nocond(self, ctx:Python3Parser.Test_nocondContext):
        pass


    # Enter a parse tree produced by Python3Parser#lambdef.
    def enterLambdef(self, ctx:Python3Parser.LambdefContext):
        pass

    # Exit a parse tree produced by Python3Parser#lambdef.
    def exitLambdef(self, ctx:Python3Parser.LambdefContext):
        pass


    # Enter a parse tree produced by Python3Parser#lambdef_nocond.
    def enterLambdef_nocond(self, ctx:Python3Parser.Lambdef_nocondContext):
        pass

    # Exit a parse tree produced by Python3Parser#lambdef_nocond.
    def exitLambdef_nocond(self, ctx:Python3Parser.Lambdef_nocondContext):
        pass


    # Enter a parse tree produced by Python3Parser#or_test.
    def enterOr_test(self, ctx:Python3Parser.Or_testContext):
        pass

    # Exit a parse tree produced by Python3Parser#or_test.
    def exitOr_test(self, ctx:Python3Parser.Or_testContext):
        pass


    # Enter a parse tree produced by Python3Parser#and_test.
    def enterAnd_test(self, ctx:Python3Parser.And_testContext):
        pass

    # Exit a parse tree produced by Python3Parser#and_test.
    def exitAnd_test(self, ctx:Python3Parser.And_testContext):
        pass


    # Enter a parse tree produced by Python3Parser#not_test.
    def enterNot_test(self, ctx:Python3Parser.Not_testContext):
        pass

    # Exit a parse tree produced by Python3Parser#not_test.
    def exitNot_test(self, ctx:Python3Parser.Not_testContext):
        pass


    # Enter a parse tree produced by Python3Parser#comparison.
    def enterComparison(self, ctx:Python3Parser.ComparisonContext):
        pass

    # Exit a parse tree produced by Python3Parser#comparison.
    def exitComparison(self, ctx:Python3Parser.ComparisonContext):
        pass


    # Enter a parse tree produced by Python3Parser#comp_op.
    def enterComp_op(self, ctx:Python3Parser.Comp_opContext):
        pass

    # Exit a parse tree produced by Python3Parser#comp_op.
    def exitComp_op(self, ctx:Python3Parser.Comp_opContext):
        pass


    # Enter a parse tree produced by Python3Parser#star_expr.
    def enterStar_expr(self, ctx:Python3Parser.Star_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#star_expr.
    def exitStar_expr(self, ctx:Python3Parser.Star_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#expr.
    def enterExpr(self, ctx:Python3Parser.ExprContext):
        pass

    # Exit a parse tree produced by Python3Parser#expr.
    def exitExpr(self, ctx:Python3Parser.ExprContext):
        pass


    # Enter a parse tree produced by Python3Parser#xor_expr.
    def enterXor_expr(self, ctx:Python3Parser.Xor_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#xor_expr.
    def exitXor_expr(self, ctx:Python3Parser.Xor_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#and_expr.
    def enterAnd_expr(self, ctx:Python3Parser.And_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#and_expr.
    def exitAnd_expr(self, ctx:Python3Parser.And_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#shift_expr.
    def enterShift_expr(self, ctx:Python3Parser.Shift_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#shift_expr.
    def exitShift_expr(self, ctx:Python3Parser.Shift_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#arith_expr.
    def enterArith_expr(self, ctx:Python3Parser.Arith_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#arith_expr.
    def exitArith_expr(self, ctx:Python3Parser.Arith_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#term.
    def enterTerm(self, ctx:Python3Parser.TermContext):
        pass

    # Exit a parse tree produced by Python3Parser#term.
    def exitTerm(self, ctx:Python3Parser.TermContext):
        pass


    # Enter a parse tree produced by Python3Parser#factor.
    def enterFactor(self, ctx:Python3Parser.FactorContext):
        pass

    # Exit a parse tree produced by Python3Parser#factor.
    def exitFactor(self, ctx:Python3Parser.FactorContext):
        pass


    # Enter a parse tree produced by Python3Parser#power.
    def enterPower(self, ctx:Python3Parser.PowerContext):
        pass

    # Exit a parse tree produced by Python3Parser#power.
    def exitPower(self, ctx:Python3Parser.PowerContext):
        pass


    # Enter a parse tree produced by Python3Parser#atom_expr.
    def enterAtom_expr(self, ctx:Python3Parser.Atom_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#atom_expr.
    def exitAtom_expr(self, ctx:Python3Parser.Atom_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#atom.
    def enterAtom(self, ctx:Python3Parser.AtomContext):
        pass

    # Exit a parse tree produced by Python3Parser#atom.
    def exitAtom(self, ctx:Python3Parser.AtomContext):
        pass


    # Enter a parse tree produced by Python3Parser#name.
    def enterName(self, ctx:Python3Parser.NameContext):
        pass

    # Exit a parse tree produced by Python3Parser#name.
    def exitName(self, ctx:Python3Parser.NameContext):
        pass


    # Enter a parse tree produced by Python3Parser#testlist_comp.
    def enterTestlist_comp(self, ctx:Python3Parser.Testlist_compContext):
        pass

    # Exit a parse tree produced by Python3Parser#testlist_comp.
    def exitTestlist_comp(self, ctx:Python3Parser.Testlist_compContext):
        pass


    # Enter a parse tree produced by Python3Parser#trailer.
    def enterTrailer(self, ctx:Python3Parser.TrailerContext):
        pass

    # Exit a parse tree produced by Python3Parser#trailer.
    def exitTrailer(self, ctx:Python3Parser.TrailerContext):
        pass


    # Enter a parse tree produced by Python3Parser#subscriptlist.
    def enterSubscriptlist(self, ctx:Python3Parser.SubscriptlistContext):
        pass

    # Exit a parse tree produced by Python3Parser#subscriptlist.
    def exitSubscriptlist(self, ctx:Python3Parser.SubscriptlistContext):
        pass


    # Enter a parse tree produced by Python3Parser#subscript_.
    def enterSubscript_(self, ctx:Python3Parser.Subscript_Context):
        pass

    # Exit a parse tree produced by Python3Parser#subscript_.
    def exitSubscript_(self, ctx:Python3Parser.Subscript_Context):
        pass


    # Enter a parse tree produced by Python3Parser#sliceop.
    def enterSliceop(self, ctx:Python3Parser.SliceopContext):
        pass

    # Exit a parse tree produced by Python3Parser#sliceop.
    def exitSliceop(self, ctx:Python3Parser.SliceopContext):
        pass


    # Enter a parse tree produced by Python3Parser#exprlist.
    def enterExprlist(self, ctx:Python3Parser.ExprlistContext):
        pass

    # Exit a parse tree produced by Python3Parser#exprlist.
    def exitExprlist(self, ctx:Python3Parser.ExprlistContext):
        pass


    # Enter a parse tree produced by Python3Parser#testlist.
    def enterTestlist(self, ctx:Python3Parser.TestlistContext):
        pass

    # Exit a parse tree produced by Python3Parser#testlist.
    def exitTestlist(self, ctx:Python3Parser.TestlistContext):
        pass


    # Enter a parse tree produced by Python3Parser#dictorsetmaker.
    def enterDictorsetmaker(self, ctx:Python3Parser.DictorsetmakerContext):
        pass

    # Exit a parse tree produced by Python3Parser#dictorsetmaker.
    def exitDictorsetmaker(self, ctx:Python3Parser.DictorsetmakerContext):
        pass


    # Enter a parse tree produced by Python3Parser#classdef.
    def enterClassdef(self, ctx:Python3Parser.ClassdefContext):
        pass

    # Exit a parse tree produced by Python3Parser#classdef.
    def exitClassdef(self, ctx:Python3Parser.ClassdefContext):
        pass


    # Enter a parse tree produced by Python3Parser#arglist.
    def enterArglist(self, ctx:Python3Parser.ArglistContext):
        pass

    # Exit a parse tree produced by Python3Parser#arglist.
    def exitArglist(self, ctx:Python3Parser.ArglistContext):
        pass


    # Enter a parse tree produced by Python3Parser#argument.
    def enterArgument(self, ctx:Python3Parser.ArgumentContext):
        pass

    # Exit a parse tree produced by Python3Parser#argument.
    def exitArgument(self, ctx:Python3Parser.ArgumentContext):
        pass


    # Enter a parse tree produced by Python3Parser#comp_iter.
    def enterComp_iter(self, ctx:Python3Parser.Comp_iterContext):
        pass

    # Exit a parse tree produced by Python3Parser#comp_iter.
    def exitComp_iter(self, ctx:Python3Parser.Comp_iterContext):
        pass


    # Enter a parse tree produced by Python3Parser#comp_for.
    def enterComp_for(self, ctx:Python3Parser.Comp_forContext):
        pass

    # Exit a parse tree produced by Python3Parser#comp_for.
    def exitComp_for(self, ctx:Python3Parser.Comp_forContext):
        pass


    # Enter a parse tree produced by Python3Parser#comp_if.
    def enterComp_if(self, ctx:Python3Parser.Comp_ifContext):
        pass

    # Exit a parse tree produced by Python3Parser#comp_if.
    def exitComp_if(self, ctx:Python3Parser.Comp_ifContext):
        pass


    # Enter a parse tree produced by Python3Parser#encoding_decl.
    def enterEncoding_decl(self, ctx:Python3Parser.Encoding_declContext):
        pass

    # Exit a parse tree produced by Python3Parser#encoding_decl.
    def exitEncoding_decl(self, ctx:Python3Parser.Encoding_declContext):
        pass


    # Enter a parse tree produced by Python3Parser#yield_expr.
    def enterYield_expr(self, ctx:Python3Parser.Yield_exprContext):
        pass

    # Exit a parse tree produced by Python3Parser#yield_expr.
    def exitYield_expr(self, ctx:Python3Parser.Yield_exprContext):
        pass


    # Enter a parse tree produced by Python3Parser#yield_arg.
    def enterYield_arg(self, ctx:Python3Parser.Yield_argContext):
        pass

    # Exit a parse tree produced by Python3Parser#yield_arg.
    def exitYield_arg(self, ctx:Python3Parser.Yield_argContext):
        pass


    # Enter a parse tree produced by Python3Parser#strings.
    def enterStrings(self, ctx:Python3Parser.StringsContext):
        pass

    # Exit a parse tree produced by Python3Parser#strings.
    def exitStrings(self, ctx:Python3Parser.StringsContext):
        pass



del Python3Parser



from dataclasses import dataclass

from pyutplugins.plugintypes.BaseRequestResponse import BaseRequestResponse


@dataclass
class SingleFileRequestResponse(BaseRequestResponse):
    fileName: str = ''




from typing import List
from typing import cast

from logging import Logger
from logging import getLogger

from os import sep as osSep

from wx import BeginBusyCursor
from wx import EndBusyCursor as wxEndBusyCursor
from wx import Yield as wxYield

from pyutplugins.IPluginAdapter import IPluginAdapter
from pyutplugins.plugininterfaces.IOPluginInterface import IOPluginInterface

from pyutplugins.ExternalTypes import OglObjects
from pyutplugins.ExternalTypes import OglClasses
from pyutplugins.ExternalTypes import OglLinks

from pyutplugins.plugintypes.ExportDirectoryResponse import ExportDirectoryResponse
from pyutplugins.plugintypes.MultipleFileRequestResponse import MultipleFileRequestResponse
from pyutplugins.plugintypes.InputFormat import InputFormat
from pyutplugins.plugintypes.OutputFormat import OutputFormat

from pyutplugins.plugintypes.PluginDataTypes import PluginDescription
from pyutplugins.plugintypes.PluginDataTypes import PluginExtension
from pyutplugins.plugintypes.PluginDataTypes import FormatName
from pyutplugins.plugintypes.PluginDataTypes import PluginName

from pyutplugins.ioplugins.java.JavaReader import JavaReader
from pyutplugins.ioplugins.java.JavaWriter import JavaWriter

FORMAT_NAME:        FormatName        = FormatName("Java")
PLUGIN_EXTENSION:   PluginExtension   = PluginExtension('java')
PLUGIN_DESCRIPTION: PluginDescription = PluginDescription('Java file format')


class IOJava(IOPluginInterface):
    """
    Plugin that can
        * Write Java code from Ogl Classes in a UML Diagram
        * Read Java to and transform to a UML Diagram

    In the original implementation these were two different I/O Plugins
    """

    def __init__(self, pluginAdapter: IPluginAdapter):

        self.logger: Logger = getLogger(__name__)

        super().__init__(pluginAdapter)

        # from super class
        self._name    = PluginName('Java Code Reader and Writer')
        self._author  = "C.Dutoit <dutoitc@hotmail.com> and N. Dubois <nicdub@gmx.ch"
        self._version = '1.0'
        self._inputFormat  = InputFormat(formatName=FORMAT_NAME, extension=PLUGIN_EXTENSION, description=PLUGIN_DESCRIPTION)
        self._outputFormat = OutputFormat(formatName=FORMAT_NAME, extension=PLUGIN_EXTENSION, description=PLUGIN_DESCRIPTION)

        self._exportDirectoryName: str         = ''
        self._importDirectoryName: str         = ''
        self._filesToImport:       List['str'] = []

    def setImportOptions(self) -> bool:

        response: MultipleFileRequestResponse = self.askToImportMultipleFiles(startDirectory=None)
        if response.cancelled is True:
            return False
        else:
            self._importDirectoryName = response.directoryName
            self._filesToImport   = response.fileList

        return True

    def setExportOptions(self) -> bool:
        response: ExportDirectoryResponse = self.askForExportDirectoryName(preferredDefaultPath=None)
        if response.cancelled is True:
            return False
        else:
            self._exportDirectoryName = response.directoryName

        return True

    def read(self) -> bool:
        BeginBusyCursor()
        wxYield()

        directoryName: str        = self._importDirectoryName
        importFiles:   List[str]  = self._filesToImport
        javaReader:    JavaReader = JavaReader()

        for importFile in importFiles:
            fqFileName: str = f'{directoryName}{osSep}{importFile}'
            javaReader.parseFile(fqFileName)

        assert javaReader.reversedClasses is not None, 'Something is broken'

        oglClasses: OglClasses = OglClasses(list(javaReader.reversedClasses.values()))
        self._layoutUmlClasses(oglClasses=oglClasses)
        self._layoutLinks(oglLinks=cast(OglLinks, javaReader.reversedLinks))

        wxEndBusyCursor()
        self._pluginAdapter.indicatePluginModifiedProject()
        return True

    def write(self, oglObjects: OglObjects):

        directoryName: str        = self._exportDirectoryName
        javaWriter:    JavaWriter = JavaWriter(writeDirectory=directoryName)

        javaWriter.write(oglObjects=oglObjects)




from logging import Logger
from logging import getLogger

from wx import CANCEL
from wx import DEFAULT_DIALOG_STYLE
from wx import EVT_BUTTON
from wx import EVT_CLOSE
from wx import ID_CANCEL
from wx import ID_OK
from wx import OK
from wx import RESIZE_BORDER
from wx import STAY_ON_TOP

from wx import CommandEvent
from wx import ColourDatabase
from wx import StdDialogButtonSizer
from wx import TextCtrl
from wx import Colour

from wx.lib.sized_controls import SizedDialog


class BaseEditDialog(SizedDialog):
    baseDlgLogger: Logger = getLogger(__name__)
    """
    Provides a common place to host duplicate code
    TODO:  This is a duplicate of Pyut's dialog of the same name
    """
    def __init__(self, parent, title='', style: int = RESIZE_BORDER | STAY_ON_TOP | DEFAULT_DIALOG_STYLE):

        super().__init__(parent, title=title, style=style)

    def _layoutStandardOkCancelButtonSizer(self):
        """
        Call this last when creating controls;  Will take care of
        adding callbacks for the Ok and Cancel buttons
        """
        buttSizer: StdDialogButtonSizer = self.CreateStdDialogButtonSizer(OK | CANCEL)

        self.SetButtonSizer(buttSizer)
        self.Bind(EVT_BUTTON, self._onOk,    id=ID_OK)
        self.Bind(EVT_BUTTON, self._onClose, id=ID_CANCEL)
        self.Bind(EVT_CLOSE,  self._onClose)

    def _convertNone(self, checkString: str):
        """

        Args:
            checkString:  the string to possibly convert

        Returns: the same string, if string = None, return an empty string.
        """
        if checkString is None:
            checkString = ''
        return checkString

    # noinspection PyUnusedLocal
    def _onOk(self, event: CommandEvent):
        """
        """
        self.EndModal(OK)

    # noinspection PyUnusedLocal
    def _onClose(self, event: CommandEvent):
        """
        """
        self.EndModal(CANCEL)

    def _indicateEmptyTextCtrl(self, name: TextCtrl):

        self.baseDlgLogger.warning(f'Name is empty!!')
        name.BackgroundColour = ColourDatabase().Find('Red')

    def _indicateNonEmptyTextCtrl(self, name: TextCtrl, normalBackgroundColor: Colour):
        name.BackgroundColour = normalBackgroundColor



# Generated from Python3Parser.g4 by ANTLR 4.11.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .Python3Parser import Python3Parser
else:
    from pyutplugins.ioplugins.python.pyantlrparser.Python3Parser import Python3Parser

# This class defines a complete generic visitor for a parse tree produced by Python3Parser.

class Python3ParserVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by Python3Parser#single_input.
    def visitSingle_input(self, ctx:Python3Parser.Single_inputContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#file_input.
    def visitFile_input(self, ctx:Python3Parser.File_inputContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#eval_input.
    def visitEval_input(self, ctx:Python3Parser.Eval_inputContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#decorator.
    def visitDecorator(self, ctx:Python3Parser.DecoratorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#decorators.
    def visitDecorators(self, ctx:Python3Parser.DecoratorsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#decorated.
    def visitDecorated(self, ctx:Python3Parser.DecoratedContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#async_funcdef.
    def visitAsync_funcdef(self, ctx:Python3Parser.Async_funcdefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#funcdef.
    def visitFuncdef(self, ctx:Python3Parser.FuncdefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#parameters.
    def visitParameters(self, ctx:Python3Parser.ParametersContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#typedargslist.
    def visitTypedargslist(self, ctx:Python3Parser.TypedargslistContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#tfpdef.
    def visitTfpdef(self, ctx:Python3Parser.TfpdefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#varargslist.
    def visitVarargslist(self, ctx:Python3Parser.VarargslistContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#vfpdef.
    def visitVfpdef(self, ctx:Python3Parser.VfpdefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#stmt.
    def visitStmt(self, ctx:Python3Parser.StmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#simple_stmts.
    def visitSimple_stmts(self, ctx:Python3Parser.Simple_stmtsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#simple_stmt.
    def visitSimple_stmt(self, ctx:Python3Parser.Simple_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#expr_stmt.
    def visitExpr_stmt(self, ctx:Python3Parser.Expr_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#annassign.
    def visitAnnassign(self, ctx:Python3Parser.AnnassignContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#testlist_star_expr.
    def visitTestlist_star_expr(self, ctx:Python3Parser.Testlist_star_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#augassign.
    def visitAugassign(self, ctx:Python3Parser.AugassignContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#del_stmt.
    def visitDel_stmt(self, ctx:Python3Parser.Del_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#pass_stmt.
    def visitPass_stmt(self, ctx:Python3Parser.Pass_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#flow_stmt.
    def visitFlow_stmt(self, ctx:Python3Parser.Flow_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#break_stmt.
    def visitBreak_stmt(self, ctx:Python3Parser.Break_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#continue_stmt.
    def visitContinue_stmt(self, ctx:Python3Parser.Continue_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#return_stmt.
    def visitReturn_stmt(self, ctx:Python3Parser.Return_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#yield_stmt.
    def visitYield_stmt(self, ctx:Python3Parser.Yield_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#raise_stmt.
    def visitRaise_stmt(self, ctx:Python3Parser.Raise_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#import_stmt.
    def visitImport_stmt(self, ctx:Python3Parser.Import_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#import_name.
    def visitImport_name(self, ctx:Python3Parser.Import_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#import_from.
    def visitImport_from(self, ctx:Python3Parser.Import_fromContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#import_as_name.
    def visitImport_as_name(self, ctx:Python3Parser.Import_as_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#dotted_as_name.
    def visitDotted_as_name(self, ctx:Python3Parser.Dotted_as_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#import_as_names.
    def visitImport_as_names(self, ctx:Python3Parser.Import_as_namesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#dotted_as_names.
    def visitDotted_as_names(self, ctx:Python3Parser.Dotted_as_namesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#dotted_name.
    def visitDotted_name(self, ctx:Python3Parser.Dotted_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#global_stmt.
    def visitGlobal_stmt(self, ctx:Python3Parser.Global_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#nonlocal_stmt.
    def visitNonlocal_stmt(self, ctx:Python3Parser.Nonlocal_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#assert_stmt.
    def visitAssert_stmt(self, ctx:Python3Parser.Assert_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#compound_stmt.
    def visitCompound_stmt(self, ctx:Python3Parser.Compound_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#async_stmt.
    def visitAsync_stmt(self, ctx:Python3Parser.Async_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#if_stmt.
    def visitIf_stmt(self, ctx:Python3Parser.If_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#while_stmt.
    def visitWhile_stmt(self, ctx:Python3Parser.While_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#for_stmt.
    def visitFor_stmt(self, ctx:Python3Parser.For_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#try_stmt.
    def visitTry_stmt(self, ctx:Python3Parser.Try_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#with_stmt.
    def visitWith_stmt(self, ctx:Python3Parser.With_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#with_item.
    def visitWith_item(self, ctx:Python3Parser.With_itemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#except_clause.
    def visitExcept_clause(self, ctx:Python3Parser.Except_clauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#block.
    def visitBlock(self, ctx:Python3Parser.BlockContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#match_stmt.
    def visitMatch_stmt(self, ctx:Python3Parser.Match_stmtContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#subject_expr.
    def visitSubject_expr(self, ctx:Python3Parser.Subject_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#star_named_expressions.
    def visitStar_named_expressions(self, ctx:Python3Parser.Star_named_expressionsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#star_named_expression.
    def visitStar_named_expression(self, ctx:Python3Parser.Star_named_expressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#case_block.
    def visitCase_block(self, ctx:Python3Parser.Case_blockContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#guard.
    def visitGuard(self, ctx:Python3Parser.GuardContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#patterns.
    def visitPatterns(self, ctx:Python3Parser.PatternsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#pattern.
    def visitPattern(self, ctx:Python3Parser.PatternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#as_pattern.
    def visitAs_pattern(self, ctx:Python3Parser.As_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#or_pattern.
    def visitOr_pattern(self, ctx:Python3Parser.Or_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#closed_pattern.
    def visitClosed_pattern(self, ctx:Python3Parser.Closed_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#literal_pattern.
    def visitLiteral_pattern(self, ctx:Python3Parser.Literal_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#literal_expr.
    def visitLiteral_expr(self, ctx:Python3Parser.Literal_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#complex_number.
    def visitComplex_number(self, ctx:Python3Parser.Complex_numberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#signed_number.
    def visitSigned_number(self, ctx:Python3Parser.Signed_numberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#signed_real_number.
    def visitSigned_real_number(self, ctx:Python3Parser.Signed_real_numberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#real_number.
    def visitReal_number(self, ctx:Python3Parser.Real_numberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#imaginary_number.
    def visitImaginary_number(self, ctx:Python3Parser.Imaginary_numberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#capture_pattern.
    def visitCapture_pattern(self, ctx:Python3Parser.Capture_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#pattern_capture_target.
    def visitPattern_capture_target(self, ctx:Python3Parser.Pattern_capture_targetContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#wildcard_pattern.
    def visitWildcard_pattern(self, ctx:Python3Parser.Wildcard_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#value_pattern.
    def visitValue_pattern(self, ctx:Python3Parser.Value_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#attr.
    def visitAttr(self, ctx:Python3Parser.AttrContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#name_or_attr.
    def visitName_or_attr(self, ctx:Python3Parser.Name_or_attrContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#group_pattern.
    def visitGroup_pattern(self, ctx:Python3Parser.Group_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#sequence_pattern.
    def visitSequence_pattern(self, ctx:Python3Parser.Sequence_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#open_sequence_pattern.
    def visitOpen_sequence_pattern(self, ctx:Python3Parser.Open_sequence_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#maybe_sequence_pattern.
    def visitMaybe_sequence_pattern(self, ctx:Python3Parser.Maybe_sequence_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#maybe_star_pattern.
    def visitMaybe_star_pattern(self, ctx:Python3Parser.Maybe_star_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#star_pattern.
    def visitStar_pattern(self, ctx:Python3Parser.Star_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#mapping_pattern.
    def visitMapping_pattern(self, ctx:Python3Parser.Mapping_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#items_pattern.
    def visitItems_pattern(self, ctx:Python3Parser.Items_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#key_value_pattern.
    def visitKey_value_pattern(self, ctx:Python3Parser.Key_value_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#double_star_pattern.
    def visitDouble_star_pattern(self, ctx:Python3Parser.Double_star_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#class_pattern.
    def visitClass_pattern(self, ctx:Python3Parser.Class_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#positional_patterns.
    def visitPositional_patterns(self, ctx:Python3Parser.Positional_patternsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#keyword_patterns.
    def visitKeyword_patterns(self, ctx:Python3Parser.Keyword_patternsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#keyword_pattern.
    def visitKeyword_pattern(self, ctx:Python3Parser.Keyword_patternContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#test.
    def visitTest(self, ctx:Python3Parser.TestContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#test_nocond.
    def visitTest_nocond(self, ctx:Python3Parser.Test_nocondContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#lambdef.
    def visitLambdef(self, ctx:Python3Parser.LambdefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#lambdef_nocond.
    def visitLambdef_nocond(self, ctx:Python3Parser.Lambdef_nocondContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#or_test.
    def visitOr_test(self, ctx:Python3Parser.Or_testContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#and_test.
    def visitAnd_test(self, ctx:Python3Parser.And_testContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#not_test.
    def visitNot_test(self, ctx:Python3Parser.Not_testContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#comparison.
    def visitComparison(self, ctx:Python3Parser.ComparisonContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#comp_op.
    def visitComp_op(self, ctx:Python3Parser.Comp_opContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#star_expr.
    def visitStar_expr(self, ctx:Python3Parser.Star_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#expr.
    def visitExpr(self, ctx:Python3Parser.ExprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#xor_expr.
    def visitXor_expr(self, ctx:Python3Parser.Xor_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#and_expr.
    def visitAnd_expr(self, ctx:Python3Parser.And_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#shift_expr.
    def visitShift_expr(self, ctx:Python3Parser.Shift_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#arith_expr.
    def visitArith_expr(self, ctx:Python3Parser.Arith_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#term.
    def visitTerm(self, ctx:Python3Parser.TermContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#factor.
    def visitFactor(self, ctx:Python3Parser.FactorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#power.
    def visitPower(self, ctx:Python3Parser.PowerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#atom_expr.
    def visitAtom_expr(self, ctx:Python3Parser.Atom_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#atom.
    def visitAtom(self, ctx:Python3Parser.AtomContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#name.
    def visitName(self, ctx:Python3Parser.NameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#testlist_comp.
    def visitTestlist_comp(self, ctx:Python3Parser.Testlist_compContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#trailer.
    def visitTrailer(self, ctx:Python3Parser.TrailerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#subscriptlist.
    def visitSubscriptlist(self, ctx:Python3Parser.SubscriptlistContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#subscript_.
    def visitSubscript_(self, ctx:Python3Parser.Subscript_Context):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#sliceop.
    def visitSliceop(self, ctx:Python3Parser.SliceopContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#exprlist.
    def visitExprlist(self, ctx:Python3Parser.ExprlistContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#testlist.
    def visitTestlist(self, ctx:Python3Parser.TestlistContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#dictorsetmaker.
    def visitDictorsetmaker(self, ctx:Python3Parser.DictorsetmakerContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#classdef.
    def visitClassdef(self, ctx:Python3Parser.ClassdefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#arglist.
    def visitArglist(self, ctx:Python3Parser.ArglistContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#argument.
    def visitArgument(self, ctx:Python3Parser.ArgumentContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#comp_iter.
    def visitComp_iter(self, ctx:Python3Parser.Comp_iterContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#comp_for.
    def visitComp_for(self, ctx:Python3Parser.Comp_forContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#comp_if.
    def visitComp_if(self, ctx:Python3Parser.Comp_ifContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#encoding_decl.
    def visitEncoding_decl(self, ctx:Python3Parser.Encoding_declContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#yield_expr.
    def visitYield_expr(self, ctx:Python3Parser.Yield_exprContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#yield_arg.
    def visitYield_arg(self, ctx:Python3Parser.Yield_argContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by Python3Parser#strings.
    def visitStrings(self, ctx:Python3Parser.StringsContext):
        return self.visitChildren(ctx)



del Python3Parser


from typing import Tuple

from pyutplugins.toolplugins.sugiyama.SugiyamaNode import SugiyamaNode


class VirtualSugiyamaNode(SugiyamaNode):
    """
    VirtualSugiyamaNode: a virtual node is a node on the Sugiyama graph which
    will not be visible on the diagram.  It is used to reserve space
    for the links drawing.
    """
    def __init__(self):
        super().__init__()

        self.__position: Tuple[int, int] = (0, 0)
        self.__size:     Tuple[int, int] = (1, 1)

    def setSize(self, width: int, height: int):
        """
        Set the size of the node.

        Args:
            width: Size
            height: Size
        """
        self.__size = (width, height)

    def getSize(self) -> Tuple[int, int]:
        """
        Get size of node.

        Returns: A tuple (int, int) : (width, height)

        """
        return self.__size

    def setPosition(self, x: int, y: int):
        """
        Set node position.

        Args:
            x: position in absolute coordinates
            y: position in absolute coordinates
        """
        self.__position = (x, y)

    def getPosition(self) -> Tuple[int, int]:
        """
        Get node position.

        Returns:    (int, int) : (x, y) in absolute coordinate
        """
        return self.__position

    def __repr__(self):
        return f'VirtualSugiyamaNode level: {self.getLevel()}'




from typing import cast

from logging import Logger
from logging import getLogger

from ogl.OglObject import OglObject

from pyutplugins.ExternalTypes import FrameInformation
from pyutplugins.ExternalTypes import OglObjects

from pyutplugins.plugininterfaces.IOPluginInterface import IPluginAdapter
from pyutplugins.plugininterfaces.ToolPluginInterface import ToolPluginInterface

from pyutplugins.plugintypes.PluginDataTypes import PluginName


class ToolTransforms(ToolPluginInterface):
    """
     A plugin for making transformations : translation, rotations, ...

    TODO: Explore parameterizing x transform and adding other transforms
    """
    def __init__(self, pluginAdapter: IPluginAdapter):

        super().__init__(pluginAdapter=pluginAdapter)

        self.logger: Logger = getLogger(__name__)

        self._name      = PluginName('Transformations')
        self._author    = 'C.Dutoit'
        self._version   = '1.1'

        self._menuTitle = 'Transformations'

    def setOptions(self) -> bool:
        return True

    def doAction(self):
        # self._pluginAdapter.getSelectedOglObjects(callback=self._stashSelectedObjects)
        self._pluginAdapter.getFrameInformation(callback=self._doAction)

    def _doAction(self, frameInformation: FrameInformation):

        selectedObjects: OglObjects = frameInformation.selectedOglObjects

        frameW: int = frameInformation.frameSize.width
        frameH: int = frameInformation.frameSize.height
        # (frameW, frameH) = self._pluginAdapter.umlFrame.GetSize()
        self.logger.warning(f'frameW: {frameW} - frameH: {frameH}')

        for obj in selectedObjects:
            oglObject: OglObject = cast(OglObject, obj)
            x, y = oglObject.GetPosition()
            newX: int = frameW - x
            self.logger.info(f"x,y: {x},{y} - newX: {newX}")
            oglObject.SetPosition(newX, y)

        self._pluginAdapter.indicatePluginModifiedProject()
        self._pluginAdapter.refreshFrame()



from typing import cast
from typing import Dict
from typing import NewType

from dataclasses import dataclass

from ogl.OglClass import OglClass

from pyutmodel.PyutClass import PyutClass

from pyutplugins.common.ElementTreeData import ElementTreeData

ClassTree  = NewType('ClassTree',  Dict[str, ElementTreeData])    # string is ClassName


@dataclass
class ClassPair:

    pyutClass: PyutClass = cast(PyutClass, None)
    oglClass:  OglClass  = cast(OglClass, None)




import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()
LICENSE = (HERE / 'LICENSE').read_text()

setup(
    name="pyutplugins",
    version="0.8.41",
    author='Humberto A. Sanchez II',
    author_email='humberto.a.sanchez.ii@gmail.com',
    maintainer='Humberto A. Sanchez II',
    maintainer_email='humberto.a.sanchez.ii@gmail.com',
    description='Pyut Plugins',
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/hasii2011/pyutplugins",
    package_data={
        'pyutplugins':                       ['py.typed'],
        'pyutplugins.common':                ['py.typed'],
        'pyutplugins.common.ui':             ['py.typed'],
        'pyutplugins.common.ui,preferences': ['py.typed'],
        'pyutplugins.exceptions':                    ['py.typed'],
        'pyutplugins.ioplugins':                      ['py.typed'],
        'pyutplugins.ioplugins.dtd':                  ['py.typed'],
        'pyutplugins.ioplugins.gml':                  ['py.typed'],
        'pyutplugins.ioplugins.java':                 ['py.typed'],
        'pyutplugins.ioplugins.mermaid':              ['py.typed'],
        'pyutplugins.ioplugins.pdf':                  ['py.typed'],
        'pyutplugins.ioplugins.python':               ['py.typed'],
        'pyutplugins.ioplugins.python.pyantlrparser': ['py.typed'],
        'pyutplugins.ioplugins.wximage':              ['py.typed'],
        'pyutplugins.plugininterfaces':               ['py.typed'],
        'pyutplugins.plugintypes':                    ['py.typed'],
        'pyutplugins.preferences':                    ['py.typed'],
        'pyutplugins.toolplugins':                    ['py.typed'],
        'pyutplugins.toolplugins.orthogonal':         ['py.typed'],
        'pyutplugins.toolplugins.sugiyama':           ['py.typed'],
    },
    packages=[
        'pyutplugins', 'pyutplugins.common', 'pyutplugins.common.ui', 'pyutplugins.common.ui.preferences',
        'pyutplugins.exceptions',
        'pyutplugins.ioplugins', 'pyutplugins.ioplugins.dtd', 'pyutplugins.ioplugins.gml',  'pyutplugins.ioplugins.java',
        'pyutplugins.ioplugins.mermaid', 'pyutplugins.ioplugins.pdf', 'pyutplugins.ioplugins.python', 'pyutplugins.ioplugins.python.pyantlrparser',
        'pyutplugins.ioplugins.wximage',
        'pyutplugins.plugininterfaces',
        'pyutplugins.plugintypes',
        'pyutplugins.preferences',
        'pyutplugins.toolplugins', 'pyutplugins.toolplugins.orthogonal', 'pyutplugins.toolplugins.sugiyama',
    ],
    install_requires=[
                      'antlr4-python3-runtime==4.11.1',
                      'pyumldiagrams==2.30.8',
                      'networkx==3.0',
                      'orthogonal==1.1.8',
                      'wxPython~=4.2.0',
                      'hasiihelper~=0.1.0',
                      'hasiicommon~=0.1.0',
                      'pyutmodel==1.4.1',
                      'ogl==0.70.22',
                      'untanglepyut==0.6.41',
                      'oglio==0.6.0',
                      ]
)




from typing import cast

from logging import Logger
from logging import getLogger

from wx import Yield as wxYield


from pyutplugins.plugininterfaces.IOPluginInterface import IOPluginInterface
from pyutplugins.IPluginAdapter import ScreenMetrics
from pyutplugins.IPluginAdapter import IPluginAdapter

from pyutplugins.ExternalTypes import OglObjects

from pyutplugins.plugintypes.InputFormat import InputFormat
from pyutplugins.plugintypes.OutputFormat import OutputFormat
from pyutplugins.plugintypes.SingleFileRequestResponse import SingleFileRequestResponse

from pyutplugins.plugintypes.PluginDataTypes import PluginName
from pyutplugins.plugintypes.PluginDataTypes import FormatName
from pyutplugins.plugintypes.PluginDataTypes import PluginDescription
from pyutplugins.plugintypes.PluginDataTypes import PluginExtension

from pyutplugins.ioplugins.pdf.ImageFormat import ImageFormat
from pyutplugins.ioplugins.pdf.ImageOptions import ImageOptions
from pyutplugins.ioplugins.pdf.OglToPyUmlDefinition import OglToPyUmlDefinition

FORMAT_NAME:        FormatName = FormatName('PDF')
PLUGIN_EXTENSION:   PluginExtension = PluginExtension('pdf')
PLUGIN_DESCRIPTION: PluginDescription = PluginDescription('A simple PDF for UML diagrams')


class IOPdf(IOPluginInterface):
    """
    """
    def __init__(self, pluginAdapter: IPluginAdapter):
        """

        Args:
            pluginAdapter:   A class that implements IMediator
        """
        super().__init__(pluginAdapter=pluginAdapter)

        self.logger: Logger = getLogger(__name__)

        self._name    = PluginName('Output PDF')
        self._author  = "Humberto A. Sanchez II"
        self._version = '1.2'

        self._exportResponse: SingleFileRequestResponse = cast(SingleFileRequestResponse, None)

        self._inputFormat  = cast(InputFormat, None)
        self._outputFormat = OutputFormat(formatName=FORMAT_NAME, extension=PLUGIN_EXTENSION, description=PLUGIN_DESCRIPTION)

        self._imageOptions: ImageOptions = ImageOptions()

        self._imageOptions.imageFormat = ImageFormat.PDF

        self._autoSelectAll = True     # we are taking a picture of the entire diagram

    def setImportOptions(self) -> bool:
        return False

    def setExportOptions(self) -> bool:
        """
        Prepare the export.

        Returns:
            if False, the export is cancelled.
        """
        self._exportResponse = self.askForFileToExport(defaultFileName=self._pluginPreferences.pdfExportFileName)

        if self._exportResponse.cancelled is True:
            return False
        else:
            self._imageOptions.outputFileName = self._exportResponse.fileName
            return True

    def read(self) -> bool:
        return False

    def write(self, oglObjects: OglObjects):
        """
        Write data to a file;  Presumably, the file was specified on the call
        to setExportOptions

         Args:
            oglObjects:  list of exported objects

        """
        self.logger.info(f'export file name: {self._imageOptions.outputFileName}')
        wxYield()

        pluginVersion: str = self.version
        pyutVersion:   str = self._pluginAdapter.pyutVersion

        screenMetrics: ScreenMetrics = self._pluginAdapter.screenMetrics
        dpi:           int           = screenMetrics.dpiX

        oglToPdf: OglToPyUmlDefinition = OglToPyUmlDefinition(imageOptions=self._imageOptions,
                                                              dpi=dpi,
                                                              pyutVersion=pyutVersion,
                                                              pluginVersion=pluginVersion
                                                              )

        oglToPdf.toClassDefinitions(oglObjects=oglObjects)
        oglToPdf.layoutLines(oglObjects=oglObjects)
        oglToPdf.write()




from dataclasses import dataclass

from pyumldiagrams import Defaults

from pyutplugins.ioplugins.pdf.ImageFormat import ImageFormat


@dataclass
class ImageOptions:

    outputFileName: str = Defaults.DEFAULT_FILE_NAME
    """
    This is a fully qualified file name

    """
    imageWidth:     int = Defaults.DEFAULT_IMAGE_WIDTH
    imageHeight:    int = Defaults.DEFAULT_IMAGE_HEIGHT
    imageFormat:    ImageFormat = ImageFormat.PNG
    horizontalGap:  int = Defaults.DEFAULT_HORIZONTAL_GAP
    verticalGap:    int = Defaults.DEFAULT_VERTICAL_GAP





from typing import NewType
from typing import cast
from typing import Dict
from typing import Tuple

from logging import Logger
from logging import getLogger

from dataclasses import dataclass

from tempfile import gettempdir

from os import sep as osSep

from networkx import read_gml
from networkx import Graph

from orthogonal.mapping.EmbeddedTypes import Position
from orthogonal.mapping.EmbeddedTypes import Positions
from orthogonal.mapping.EmbeddedTypes import ScreenCoordinates
from orthogonal.mapping.EmbeddingToScreen import EmbeddingToScreen
from orthogonal.mapping.ScreenSize import ScreenSize

from orthogonal.topologyShapeMetric.Compaction import Compaction
from orthogonal.topologyShapeMetric.OrthogonalException import OrthogonalException
from orthogonal.topologyShapeMetric.Orthogonalization import Orthogonalization
from orthogonal.topologyShapeMetric.Planarization import Planarization

from pyutplugins.ExternalTypes import OglObjects

from pyutplugins.ioplugins.gml.GMLExporter import GMLExporter
from pyutplugins.preferences.PluginPreferences import PluginPreferences
from pyutplugins.toolplugins.orthogonal.LayoutAreaSize import LayoutAreaSize
from pyutplugins.toolplugins.orthogonal.OrthogonalAdapterException import OrthogonalAdapterException

GraphicsCoordinates = NewType('GraphicsCoordinates', Tuple[int, int])
LayoutEngineInput   = NewType('LayoutEngineInput', Dict[str, GraphicsCoordinates])

EngineCoordinates  = NewType('EngineCoordinates', Tuple[int, int])
LayoutEngineOutput = NewType('LayoutEngineOutput', Dict[str, EngineCoordinates])


@dataclass
class OglCoordinate:

    __slots__ = ['x', 'y']

    x: int
    y: int


OglCoordinates = Dict[str, OglCoordinate]


class OrthogonalAdapter:

    TEMPORARY_GML_LAYOUT_FILENAME: str = 'toOrthogonalLayoutV2.gml'

    def __init__(self, umlObjects: OglObjects):

        self.logger: Logger      = getLogger(__name__)
        gmlExporter: GMLExporter = GMLExporter()            # TODO: Fix this Yeah I know bad to share this way

        gmlExporter.translate(umlObjects=umlObjects)

        if PluginPreferences().debugTempFileLocation is True:
            self._pathToLayout = f'{OrthogonalAdapter.TEMPORARY_GML_LAYOUT_FILENAME}'
        else:
            tempDir: str = gettempdir()
            self._pathToLayout = f'{tempDir}{osSep}{OrthogonalAdapter.TEMPORARY_GML_LAYOUT_FILENAME}'

        with open(self._pathToLayout, 'w') as writer:
            writer.write(gmlExporter.gml)

        self._ets:            EmbeddingToScreen = cast(EmbeddingToScreen, None)
        self._nxGraph:        Graph             = cast(Graph, None)
        self._oglCoordinates: OglCoordinates    = cast(OglCoordinates, None)

    @property
    def oglCoordinates(self) -> OglCoordinates:
        return self._oglCoordinates

    def doLayout(self, layoutAreaSize: LayoutAreaSize):

        self._nxGraph = Graph(read_gml(self._pathToLayout))

        positions: LayoutEngineInput = self._toLayoutEngineInput(self._nxGraph)

        self.logger.info(f'Generated positions: {positions}')

        compact: Compaction = self._runLayout(nxGraph=self._nxGraph, positions=positions)

        enginePositions:   LayoutEngineOutput = compact.pos
        embeddedPositions: Positions = self._toEmbeddedPositions(enginePositions)

        screenSize: ScreenSize = ScreenSize(width=layoutAreaSize.width, height=layoutAreaSize.height)
        self._ets = EmbeddingToScreen(screenSize, embeddedPositions)
        self._oglCoordinates = self._toOglCoordinates(nxGraph=self._nxGraph)

    def _runLayout(self, nxGraph: Graph, positions: LayoutEngineInput) -> Compaction:

        try:
            planar:     Planarization     = Planarization(nxGraph, positions)
            orthogonal: Orthogonalization = Orthogonalization(planar)
            compact:    Compaction        = Compaction(orthogonal)
        except OrthogonalException as oe:
            eMsg: str = f'{oe}'
            self.logger.error(eMsg)
            raise OrthogonalAdapterException(eMsg)

        return compact

    def _toLayoutEngineInput(self, nxGraph: Graph) -> LayoutEngineInput:

        positions: LayoutEngineInput = LayoutEngineInput({})
        for node in nxGraph:
            self.logger.debug(f'node: {node}')
            x: int = nxGraph.nodes[node]['graphics']['x']
            y: int = nxGraph.nodes[node]['graphics']['y']
            positions[node] = GraphicsCoordinates((x, y))

        return positions

    def _toEmbeddedPositions(self, embeddedPositions: LayoutEngineOutput) -> Positions:

        positions: Positions = {}
        for nodeName in embeddedPositions.keys():
            ePos: EngineCoordinates = embeddedPositions[nodeName]
            position: Position = Position(x=ePos[0], y=ePos[1])

            positions[nodeName] = position

        return positions

    def _toOglCoordinates(self, nxGraph: Graph) -> OglCoordinates:

        retCoordinates: OglCoordinates = {}
        for node in nxGraph:
            scrCoordinates: ScreenCoordinates = self._ets.getScreenPosition(node)
            oglCoordinates: OglCoordinate     = OglCoordinate(scrCoordinates.x, scrCoordinates.y)

            retCoordinates[node] = oglCoordinates

        return retCoordinates



from typing import List
from typing import cast

from logging import Logger
from logging import getLogger


from wx import EVT_CHECKBOX
from wx import EVT_CHOICE
from wx import EVT_TEXT
from wx import ID_ANY

from wx import CommandEvent
from wx import StaticText
from wx import TextCtrl
from wx import Window
from wx import CheckBox
from wx import Choice

from wx import NewIdRef as wxNewIdRef

from wx.lib.sized_controls import SizedPanel
from wx.lib.sized_controls import SizedStaticBox

from pyutplugins.common.ui.Dimensions import Dimensions
from pyutplugins.common.ui.DimensionsControl import DimensionsControl
from pyutplugins.ioplugins.mermaid.MermaidDirection import MermaidDirection
from pyutplugins.preferences.PluginPreferences import PluginPreferences
from pyutplugins.toolplugins.orthogonal.LayoutAreaSize import LayoutAreaSize


class PluginPreferencesPage(SizedPanel):

    def __init__(self, parent: Window):
        self.logger: Logger = getLogger(__name__)

        super().__init__(parent)
        self.SetSizerType('vertical')

        self._preferences: PluginPreferences = PluginPreferences()
        self._pdfFileNameWxId:   int = wxNewIdRef()
        self._imageFileNameWxId: int = wxNewIdRef()

        self._layoutSizeControls:     DimensionsControl = cast(DimensionsControl, None)
        self._stepSugiyama:           CheckBox          = cast(CheckBox, None)
        self._mermaidLayoutDirection: Choice            = cast(Choice, None)

        self._createWindow(self)

        self._setControlValues()
        parent.Bind(EVT_TEXT,     self._onNameChange,  id=self._pdfFileNameWxId)
        parent.Bind(EVT_TEXT,     self._onNameChange,  id=self._imageFileNameWxId)

        parent.Bind(EVT_CHECKBOX, self._onSugiyamaValueChanged,   self._stepSugiyama)
        parent.Bind(EVT_CHOICE,   self._onLayoutDirectionChanged, self._mermaidLayoutDirection)

    @property
    def name(self) -> str:
        return 'Plugins'

    def _createWindow(self, parent: SizedPanel):

        self._stepSugiyama = CheckBox(parent, label='Step Sugiyama Layout')

        self._layoutMermaidPreferences(parent)

        self._layoutSizeControls = DimensionsControl(sizedPanel=parent, displayText="Orthogonal Layout Width/Height",
                                                     minValue=100, maxValue=4096,
                                                     valueChangedCallback=self._layoutSizeChanged,
                                                     setControlsSize=False)

        # noinspection PyUnresolvedReferences
        self._layoutSizeControls.SetSizerProps(proportion=2, expand=False)

        self._layoutNamePreferences(parent=parent)

    def _layoutMermaidPreferences(self, parent):

        directions: List[str] = [s for s in MermaidDirection]

        ssb: SizedStaticBox = SizedStaticBox(parent, label='Mermaid Diagram Layout Direction')
        ssb.SetSizerProps(proportion=2, expand=False)

        self._mermaidLayoutDirection = Choice(ssb, choices=directions)

    def _layoutNamePreferences(self, parent: SizedPanel):
        sizedForm: SizedPanel = SizedPanel(parent)
        sizedForm.SetSizerType('form')
        sizedForm.SetSizerProps(proportion=2)
        StaticText(sizedForm, ID_ANY, 'PDF Filename:')
        # TODO need plugin preference
        TextCtrl(sizedForm, id=self._pdfFileNameWxId, value=self._preferences.pdfExportFileName, size=(125, 25))

        StaticText(sizedForm, ID_ANY, 'Image Filename:')
        TextCtrl(sizedForm, self._imageFileNameWxId, self._preferences.wxImageFileName, size=(125, 25))

    def _setControlValues(self):
        layoutDimensions: Dimensions = Dimensions()
        layoutDimensions.width  = self._preferences.orthogonalLayoutSize.width
        layoutDimensions.height = self._preferences.orthogonalLayoutSize.height

        self._layoutSizeControls.dimensions = layoutDimensions

        self._stepSugiyama.SetValue(self._preferences.sugiyamaStepByStep)

    def _onNameChange(self, event: CommandEvent):

        eventID:  int = event.GetId()
        newValue: str = event.GetString()

        match eventID:
            case self._imageFileNameWxId:
                self._preferences.wxImageFileName = newValue
            case self._pdfFileNameWxId:
                self._preferences.pdfExportFileName = newValue
            case _:
                self.logger.error(f'Unknown event id')

    def _layoutSizeChanged(self, newValue: Dimensions):
        layoutAreaSize: LayoutAreaSize = LayoutAreaSize(width=newValue.width, height=newValue.height)
        self._preferences.orthogonalLayoutSize = layoutAreaSize
        self._valuesChanged = True

    def _onSugiyamaValueChanged(self, event: CommandEvent):

        newValue: bool = event.IsChecked()
        self._preferences.sugiyamaStepByStep = newValue

    # noinspection PyUnusedLocal
    def _onLayoutDirectionChanged(self, event: CommandEvent):
        idx:     int = self._mermaidLayoutDirection.GetSelection()
        enumStr: str = self._mermaidLayoutDirection.GetString(idx)

        prefValue: MermaidDirection = MermaidDirection.toEnum(enumStr=enumStr)

        self._preferences.mermaidLayoutDirection = prefValue




from abc import ABC

from pyutplugins.plugintypes.PluginDataTypes import PluginDescription
from pyutplugins.plugintypes.PluginDataTypes import PluginExtension
from pyutplugins.plugintypes.PluginDataTypes import FormatName

from pyutplugins.exceptions.InvalidPluginExtensionException import InvalidPluginExtensionException
from pyutplugins.exceptions.InvalidPluginNameException import InvalidPluginNameException

DOT:                str = '.'
SPECIAL_CHARACTERS: str = '!@#$%^&*_+-=[]{};:,.<>?/|\'\"'


class BaseFormat(ABC):
    """
    Provides the basic capabilities;  Should not be directly instantiated;
    TODO:  Figure out how to prevent that
    https://stackoverflow.com/questions/7989042/preventing-a-class-from-direct-instantiation-in-python#7990308
    If we do the above; have to figure out how to tests the base functionality in TestBaseFormat
    """
    def __init__(self, formatName: FormatName, extension: PluginExtension, description: PluginDescription):

        if self.__containsSpecialCharacters(formatName):  # TODO Must be a better way
            raise InvalidPluginNameException(f'{formatName}')

        if DOT in extension:
            raise InvalidPluginExtensionException(f'{extension}')

        self._name:        FormatName        = formatName
        self._extension:   PluginExtension   = extension
        self._description: PluginDescription = description

    @property
    def formatName(self) -> FormatName:
        """
        No special characters allowed

        Returns: The Plugin's name
        """
        return self._name

    @property
    def extension(self) -> PluginExtension:
        """
        Returns: The file name extension (w/o the leading dot '.')
        """
        return self._extension

    @property
    def description(self) -> PluginDescription:
        """
        Returns: The textual description of the plugin format
        """
        return self._description

    def __containsSpecialCharacters(self, name: FormatName) -> bool:
        for special in SPECIAL_CHARACTERS:
            if special in name:
                return True
        return False




from enum import Enum

from wx import BITMAP_TYPE_BMP
from wx import BITMAP_TYPE_JPEG
from wx import BITMAP_TYPE_PNG
from wx import BITMAP_TYPE_TIFF

# noinspection PyProtectedMember
from wx._core import BitmapType


class WxImageFormat(Enum):

    PNG  = 'png'
    JPG  = 'jpg'
    BMP  = 'bmp'
    TIFF = 'tiff'

    @staticmethod
    def toWxBitMapType(wxImageFormat: 'WxImageFormat') -> BitmapType:

        if wxImageFormat == WxImageFormat.PNG:
            return BITMAP_TYPE_PNG
        elif wxImageFormat == WxImageFormat.JPG:
            return BITMAP_TYPE_JPEG
        elif wxImageFormat == WxImageFormat.BMP:
            return BITMAP_TYPE_BMP
        elif wxImageFormat == WxImageFormat.TIFF:
            return BITMAP_TYPE_TIFF
        else:
            assert False, 'Unhandled WxImageFormat value'

    def __str__(self):
        return self.name.lower()

    def __repr__(self):
        return self.name.lower()
